@@ -1,5 +1,5 @@
 /* append.h -- Description of messages to be copied 
- * $Id: append.h,v 1.26 2004/01/22 21:17:07 ken3 Exp $ 
+ * $Id: append.h,v 1.24.2.4 2005/06/02 16:16:02 ken3 Exp $ 
  *
  * Copyright (c) 1998, 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -46,6 +46,7 @@
 #define INCLUDED_APPEND_H
 
 #include "mailbox.h"
+#include "message.h"
 #include "prot.h"
 
 struct copymsg {
@@ -59,6 +60,7 @@ struct copymsg {
     const char *cache_begin;
     int cache_len;		/* 0 if need to copy & parse message */
     int seen;
+    struct message_uuid uuid;
     bit32 system_flags;
     char *flag[MAX_USER_FLAGS+1];
 };
@@ -123,21 +125,21 @@ extern FILE *append_newstage(const char *mailboxname, time_t internaldate,
 			     int msgnum, struct stagemsg **stagep);
 
 /* adds a new mailbox to the stage initially created by append_newstage() */
-extern int append_fromstage(struct appendstate *mailbox,
+extern int append_fromstage(struct appendstate *mailbox, struct body **body,
 			    struct stagemsg *stage, time_t internaldate,
 			    const char **flag, int nflags, int nolink);
 
 /* removes the stage (frees memory, deletes the staging files) */
 extern int append_removestage(struct stagemsg *stage);
 
-extern int append_fromstream(struct appendstate *as,
+extern int append_fromstream(struct appendstate *as, struct body **body,
 			     struct protstream *messagefile,
 			     unsigned long size, time_t internaldate,
 			     const char **flag, int nflags);
 
 extern int append_copy(struct mailbox *mailbox,
 		       struct appendstate *append_mailbox,
-		       int nummsg, struct copymsg *copymsg);
+		       int nummsg, struct copymsg *copymsg, int nolink);
 
 extern int append_collectnews(struct appendstate *mailbox,
 			      const char *group, unsigned long feeduid);
