@@ -39,11 +39,12 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: backend.h,v 1.7.2.6 2005/02/21 19:25:19 ken3 Exp $ */
+/* $Id: backend.h,v 1.7.2.7 2006/01/17 20:15:20 murch Exp $ */
 
 #ifndef _INCLUDED_BACKEND_H
 #define _INCLUDED_BACKEND_H
 
+#include "global.h"
 #include "mboxlist.h"
 #include "prot.h"
 #include "protocol.h"
@@ -51,8 +52,6 @@
 
 /* Functionality to bring up/down connections to backend servers */
 
-#define LAST_RESULT_LEN 1024
-
 struct backend {
     char hostname[MAX_PARTITION_LEN];
     struct sockaddr_storage addr;
@@ -77,7 +76,7 @@ struct backend {
 
     unsigned long capability;
 
-    char last_result[LAST_RESULT_LEN];
+    struct buf last_result;
     struct protstream *in; /* from the be server to me, the proxy */
     struct protstream *out; /* to the be server */
 };
