@@ -1,6 +1,6 @@
 /* tree.c -- abstract syntax tree handling
  * Larry Greenfield
- * $Id: tree.c,v 1.8 2000/12/18 04:53:43 leg Exp $
+ * $Id: tree.c,v 1.11.2.3 2005/03/12 03:30:13 ken3 Exp $
  */
 /***********************************************************
         Copyright 1999 by Carnegie Mellon University
@@ -43,13 +43,6 @@ stringlist_t *new_sl(char *s, stringlist_t *n)
     return p;
 }
 
-patternlist_t *new_pl(void *pat, patternlist_t *n)
-{
-    patternlist_t *p = (patternlist_t *) xmalloc(sizeof(patternlist_t));
-    p->p = pat;
-    p->next = n;
-    return p;
-}
 
 tag_t *new_tag(int type, char *s)
 {
@@ -115,26 +108,6 @@ void free_sl(stringlist_t *sl)
     }
 }
 
-void free_pl(patternlist_t *pl, int comptag) 
-{
-    patternlist_t *pl2;
-
-    while (pl != NULL) {
-	pl2 = pl->next;
-
-	if (pl->p) {
-#ifdef ENABLE_REGEX
-	    if (comptag == REGEX) {
-		regfree((regex_t *) pl->p);
-	    }
-#endif
-	    free(pl->p);
-	}
-
-	free(pl);
-	pl = pl2;
-    }
-}
 
 void free_test(test_t *t);
 
@@ -173,12 +146,18 @@ void free_test(test_t *t)
 
     case HEADER:
 	free_sl(t->u.h.sl);
-	free_pl(t->u.h.pl, t->u.h.comptag);
+	free_sl(t->u.h.pl);
+	
 	break;
 
     case ADDRESS:
 	free_sl(t->u.ae.sl);
-	free_pl(t->u.ae.pl, t->u.ae.comptag);
+	free_sl(t->u.ae.pl);
+	break;
+
+    case BODY:
+	free_sl(t->u.b.content_types);
+	free_sl(t->u.b.pl);
 	break;
 
     case NOT:
@@ -202,6 +181,18 @@ void free_tree(commandlist_t *cl)
 	    free_tree(cl->u.i.do_else);
 	    break;
 
+	case INCLUDE:
+	    if (cl->u.inc.script) free(cl->u.inc.script);
+	    break;
+
+	case FILEINTO:
+	    if (cl->u.f.folder) free(cl->u.f.folder);
+	    break;
+
+	case REDIRECT:
+	    if (cl->u.r.address) free(cl->u.r.address);
+	    break;
+
 	case REJCT:
 	    if (cl->u.str) free(cl->u.str);
 	    break;
@@ -212,8 +203,6 @@ void free_tree(commandlist_t *cl)
 	    if (cl->u.v.message) free(cl->u.v.message);
 	    break;
 	    
-	case FILEINTO:
-	case FORWARD:
 	case SETFLAG:
 	case ADDFLAG:
 	case REMOVEFLAG:
@@ -223,6 +212,25 @@ void free_tree(commandlist_t *cl)
 	case KEEP:
 	case STOP:
 	case DISCARD:
+	case RETURN:
+	    break;
+
+	case NOTIFY:
+	    if (cl->u.n.method) free(cl->u.n.method);
+	    if (cl->u.n.id) free(cl->u.n.id);
+	    if (cl->u.n.options) free_sl(cl->u.n.options);
+	    if (cl->u.n.message) free(cl->u.n.message);
+	    break;
+
+	case DENOTIFY:
+	    if (cl->u.d.pattern) {
+#ifdef ENABLE_REGEX
+		if (cl->u.d.comptag == REGEX) {
+		    regfree((regex_t *) cl->u.d.pattern);
+		}
+#endif
+		free(cl->u.d.pattern);
+	    }
 	    break;
 	}
 
